@@ -11,15 +11,16 @@ import os
 import socket
 import time
 from collections import namedtuple
-from functools import partial
 from random import random
 from shutil import copyfile
-from typing import Tuple, Optional
+from typing import Any, Tuple, Optional
 from urllib.parse import urlencode, urlunparse
 
 import aiohttp
 import lxml.objectify
 import requests
+from tracing.traced_session import get_traced_session
+from tracing.traced_aiohttp import create_traced_async_session
 
 from cmapi_server.exceptions import CMAPIBasicError
 # Bug in pylint https://github.com/PyCQA/pylint/issues/4584
@@ -27,10 +28,11 @@ requests.packages.urllib3.disable_warnings()  # pylint: disable=no-member
 
 from cmapi_server.constants import (
     CMAPI_CONF_PATH, CMAPI_DEFAULT_CONF_PATH, DEFAULT_MCS_CONF_PATH,
-    DEFAULT_SM_CONF_PATH, LOCALHOSTS
+    DEFAULT_SM_CONF_PATH, LOCALHOSTS, _version
 )
 from cmapi_server.handlers.cej import CEJPasswordHandler
 from cmapi_server.managers.process import MCSProcessManager
+from cmapi_server.managers.application import AppStatefulConfig
 from mcs_node_control.models.node_config import NodeConfig
 
 
@@ -99,6 +101,7 @@ def start_transaction(
     final_time = datetime.datetime.now() + datetime.timedelta(seconds=timeout)
 
     success = False
+    successes = []
     while datetime.datetime.now() < final_time and not success:
         successes = []
 
@@ -136,6 +139,10 @@ def start_transaction(
         # this copy will be updated if an optional node can't be reached
         real_active_nodes = set(active_nodes)
         logging.trace(f'Active nodes on start transaction {active_nodes}')
+
+        if not len(active_nodes):
+            logging.warning('No active nodes found, transaction start will not have any effect')
+
         for node in active_nodes:
             url = f'https://{node}:8640/cmapi/{version}/node/begin'
             node_success = False
@@ -152,9 +159,9 @@ def start_transaction(
                     body['timeout'] = (
                         final_time - datetime.datetime.now()
                     ).seconds
-                    r = requests.put(
-                        url, verify=False, headers=headers, json=body,
-                        timeout=10
+                    r = get_traced_session().request(
+                        'PUT', url, verify=False, headers=headers,
+                        json=body, timeout=10
                     )
 
                     # a 4xx error from our endpoint;
@@ -218,8 +225,9 @@ def rollback_txn_attempt(key, version, txnid, nodes):
         url = f"https://{node}:8640/cmapi/{version}/node/rollback"
         for retry in range(5):
             try:
-                r = requests.put(
-                    url, verify=False, headers=headers, json=body, timeout=5
+                r = get_traced_session().request(
+                    'PUT', url, verify=False, headers=headers,
+                    json=body, timeout=5
                 )
                 r.raise_for_status()
             except requests.Timeout:
@@ -273,7 +281,10 @@ def commit_transaction(
         url = f"https://{node}:8640/cmapi/{version}/node/commit"
         for retry in range(5):
             try:
-                r = requests.put(url, verify = False, headers = headers, json = body, timeout = 5)
+                r = get_traced_session().request(
+                    'PUT', url, verify=False, headers=headers,
+                    json=body, timeout=5
+                )
                 r.raise_for_status()
             except requests.Timeout as e:
                 logging.warning(f"commit_transaction(): timeout on node {node}")
@@ -291,28 +302,23 @@ def broadcast_new_config(
     test_mode: bool = False,
     nodes: Optional[list] = None,
     timeout: Optional[int] = None,
-    distribute_secrets: bool = False
+    distribute_secrets: bool = False,
+    stateful_config_dict: Optional[dict[str, Any]] = None
 ) -> None:
     """Send new config to nodes. Now in async way.
 
     :param cs_config_filename: Columnstore.xml path,
                                defaults to DEFAULT_MCS_CONF_PATH
-    :type cs_config_filename: str, optional
     :param cmapi_config_filename: cmapi config path,
                                   defaults to CMAPI_CONF_PATH
-    :type cmapi_config_filename: str, optional
     :param sm_config_filename: storage manager config path,
                                defaults to DEFAULT_SM_CONF_PATH
-    :type sm_config_filename: str, optional
     :param test_mode: for test purposes, defaults to False TODO: remove
-    :type test_mode: bool, optional
     :param nodes: nodes list for config put, defaults to None
-    :type nodes: Optional[list], optional
     :param timeout: timeout passing to gracefully stop DMLProc TODO: for next
                     releases. Could affect all logic of broadcacting new config
-    :type timeout: Optional[int], optional
     :param distribute_secrets: flag to distribute secrets to nodes
-    :type distribute_secrets: bool
+    :param stateful_config_dict: stateful config update dict to distribute to nodes
     :raises CMAPIBasicError: If Broadcasting config to nodes failed with errors
     """
 
@@ -323,24 +329,32 @@ def broadcast_new_config(
     if nodes is None:
         nodes = get_active_nodes(cs_config_filename)
 
-    nc = NodeConfig()
-    root = nc.get_current_config_root(config_filename=cs_config_filename)
-    with open(cs_config_filename) as f:
-        config_text = f.read()
-
-    with open(sm_config_filename) as f:
-        sm_config_text = f.read()
-
     headers = {'x-api-key': key}
-    body = {
-        'manager': root.find('./ClusterManager').text,
-        'revision': root.find('./ConfigRevision').text,
-        'timeout': 300,
-        'config': config_text,
-        'cs_config_filename': cs_config_filename,
-        'sm_config_filename': sm_config_filename,
-        'sm_config': sm_config_text
-    }
+    if stateful_config_dict:
+        body = {
+            'timeout': 300,
+            'stateful_config_dict': stateful_config_dict,
+            'only_stateful_config': True,
+        }
+    else:
+        nc = NodeConfig()
+        root = nc.get_current_config_root(config_filename=cs_config_filename)
+        with open(cs_config_filename, mode='r', encoding='utf-8') as f:
+            config_text = f.read()
+
+        with open(sm_config_filename, mode='r', encoding='utf-8') as f:
+            sm_config_text = f.read()
+
+        body = {
+            'manager': root.find('./ClusterManager').text,
+            'revision': root.find('./ConfigRevision').text,
+            'timeout': 300,
+            'config': config_text,
+            'mcs_config_filename': cs_config_filename,
+            'sm_config_filename': sm_config_filename,
+            'sm_config': sm_config_text,
+            'stateful_config_dict': AppStatefulConfig.to_dict(),
+        }
 
     if distribute_secrets:
         # TODO: do not restart cluster when put xml config only with
@@ -359,11 +373,8 @@ def broadcast_new_config(
         """Update remote node config asyncronously.
 
         :param node: node ip address or hostname
-        :type node: str
         :param headers: headers for request
-        :type headers: dict
         :param body: request body
-        :type body: dict
         :raises CMAPIBasicError: If request failed by status code
         :raises CMAPIBasicError: If request failed by some undefined error
         :raises CMAPIBasicError: If request failed by timeout
@@ -372,14 +383,14 @@ def broadcast_new_config(
         url = f'https://{node}:8640/cmapi/{version}/node/config'
         resp_json: dict = dict()
 
-        async with aiohttp.ClientSession() as session:
+        async with create_traced_async_session() as session:
             try:
                 async with session.put(
                     url, headers=headers, json=body, ssl=False, timeout=120
                 ) as response:
                     resp_json =  await response.json(encoding='utf-8')
                     response.raise_for_status()
-                logging.info(f'Node {node} config put successfull.')
+                logging.info(f'Node {node} config put successful.')
             except aiohttp.ClientResponseError as err:
                 # TODO: may be better to check if resp status is 422 cause
                 #       it's like a signal that cmapi server raised it in
@@ -433,6 +444,27 @@ def broadcast_new_config(
         raise CMAPIBasicError(final_message)
 
 
+def broadcast_stateful_config(stateful_config_dict: dict[str, Any]) -> None:
+    """Broadcast new stateful config to nodes.
+
+    :param stateful_config_dict: stateful config update dict to distribute to nodes
+    """
+
+    try:
+        broadcast_new_config(stateful_config_dict=stateful_config_dict)
+    except CMAPIBasicError as err:
+        message = (
+            f'Failed to broadcast new stateful config dict: {stateful_config_dict}, '
+            f'got error: {err.message}'
+        )
+        logging.error(message)
+        return
+    else:
+        logging.debug(
+            f'Successfully broadcasted new stateful config dict: {stateful_config_dict}'
+        )
+
+
 # Might be more appropriate to put these in node_manipulation?
 def update_revision_and_manager(
     input_config_filename: Optional[str] = None,
@@ -566,7 +598,6 @@ def get_current_key(config_parser):
 
 
 def get_version():
-    from cmapi_server.controllers.dispatcher import _version
     return _version
 
 
@@ -577,16 +608,36 @@ def get_dbroots(node, config=DEFAULT_MCS_CONF_PATH):
     dbroots = []
     smc_node = root.find('./SystemModuleConfig')
     mod_count = int(smc_node.find('./ModuleCount3').text)
+
     for i in range(1, mod_count+1):
         ip_addr = smc_node.find(f'./ModuleIPAddr{i}-1-3').text
         hostname = smc_node.find(f'./ModuleHostName{i}-1-3').text
-        node_fqdn = socket.gethostbyaddr(hostname)[0]
+        try:
+            node_fqdn = socket.gethostbyaddr(hostname)[0]
+        except (socket.herror, socket.gaierror, OSError) as e:
+            # Fallback if reverse lookup fails
+            logging.warning(
+                'get_dbroots(): reverse lookup failed for %r: %s. Using original hostname.',
+                hostname, e
+            )
+            node_fqdn = hostname
 
         if node in LOCALHOSTS and hostname != 'localhost':
-            node = socket.gethostbyaddr(socket.gethostname())[0]
+            try:
+                node = socket.gethostbyaddr(socket.gethostname())[0]
+            except (socket.herror, socket.gaierror, OSError) as e:
+                logging.warning(
+                    'get_dbroots(): reverse lookup failed for local hostname: %s. Using socket.gethostname().',
+                    e
+                )
+                node = socket.gethostname()
         elif node not in LOCALHOSTS and hostname == 'localhost':
             # hostname will only be loclahost if we are in one node cluster
-            hostname = socket.gethostbyaddr(socket.gethostname())[0]
+            try:
+                hostname = socket.gethostbyaddr(socket.gethostname())[0]
+            except (socket.herror, socket.gaierror, OSError) as e:
+                logging.warning('get_dbroots(): reverse lookup failed for "localhost": %s.', e)
+                hostname = socket.gethostname()
 
 
         if node == ip_addr or node == hostname or node == node_fqdn:
@@ -596,6 +647,7 @@ def get_dbroots(node, config=DEFAULT_MCS_CONF_PATH):
                 dbroots.append(
                     smc_node.find(f"./ModuleDBRootID{i}-{j}-3").text
                 )
+
     return dbroots
 
 
@@ -655,7 +707,7 @@ def get_current_config_file(
         headers = {'x-api-key' : key}
         url = f'https://{node}:8640/cmapi/{get_version()}/node/config'
         try:
-            r = requests.get(url, verify=False, headers=headers, timeout=5)
+            r = get_traced_session().request('GET', url, verify=False, headers=headers, timeout=5)
             r.raise_for_status()
             config = r.json()['config']
         except Exception as e:
@@ -766,14 +818,17 @@ def if_primary_restart(
     success = False
     while not success and datetime.datetime.now() < endtime:
         try:
-            response = requests.put(url, verify = False, headers = headers, json = body, timeout = 60)
+            response = get_traced_session().request(
+                'PUT', url, verify=False, headers=headers,
+                json=body, timeout=60
+            )
             response.raise_for_status()
             success = True
         except Exception as e:
             logging.warning(f"if_primary_restart(): failed to start the cluster, got {str(e)}")
             time.sleep(10)
     if not success:
-        logging.error(f"if_primary_restart(): failed to start the cluster.  Manual intervention is required.")
+        logging.error("if_primary_restart(): failed to start the cluster.  Manual intervention is required.")
 
 
 def get_cej_info(config_root):
