@@ -21,6 +21,23 @@
 #include "ha_mcs_sysvars.h"
 #include "mcsconfig.h"
 
+#if MYSQL_VERSION_ID >= 110800
+#define DEFINE_TYPELIB(A) { \
+    array_elements(A) - 1,  \
+    #A,                     \
+    A,                      \
+    nullptr,                \
+    nullptr                 \
+  }
+#else
+#define DEFINE_TYPELIB(A) { \
+    array_elements(A) - 1,  \
+    #A,                     \
+    A,                      \
+    nullptr                 \
+  }
+#endif
+
 const char* mcs_compression_type_names[] = {"SNAPPY",  // 0
                                             "SNAPPY",  // 1
                                             "SNAPPY",  // 2
@@ -29,9 +46,7 @@ const char* mcs_compression_type_names[] = {"SNAPPY",  // 0
 #endif
                                             NullS};
 
-static TYPELIB mcs_compression_type_names_lib = {array_elements(mcs_compression_type_names) - 1,
-                                                 "mcs_compression_type_names", mcs_compression_type_names,
-                                                 NULL};
+static TYPELIB mcs_compression_type_names_lib = DEFINE_TYPELIB(mcs_compression_type_names);
 
 // compression type
 static MYSQL_THDVAR_ENUM(compression_type, PLUGIN_VAR_RQCMDARG,
@@ -39,8 +54,8 @@ static MYSQL_THDVAR_ENUM(compression_type, PLUGIN_VAR_RQCMDARG,
                          "SNAPPY segment files are Snappy compressed (default);"
 #ifdef HAVE_LZ4
                          "LZ4 segment files are LZ4 compressed;",
-# else
-			 ,
+#else
+                         ,
 #endif
                          NULL,                              // check
                          NULL,                              // update
@@ -49,23 +64,21 @@ static MYSQL_THDVAR_ENUM(compression_type, PLUGIN_VAR_RQCMDARG,
 
 // fe_conn_info pointer
 static MYSQL_THDVAR_ULONGLONG(fe_conn_info_ptr, PLUGIN_VAR_NOSYSVAR | PLUGIN_VAR_NOCMDOPT,
-                              "FrontEnd connection structure pointer. For internal usage.", NULL, NULL, 0, 0,
+                              "FrontEnd connection structure pointer. For internal usage", NULL, NULL, 0, 0,
                               ~0U, 1);
 
 // optimizer flags vault
 static MYSQL_THDVAR_ULONGLONG(original_optimizer_flags, PLUGIN_VAR_NOSYSVAR | PLUGIN_VAR_NOCMDOPT,
-                              "Vault for original optimizer flags. For internal usage.", NULL, NULL, 0, 0,
+                              "Vault for original optimizer flags. For internal usage", NULL, NULL, 0, 0,
                               ~0U, 1);
 
 static MYSQL_THDVAR_ULONGLONG(original_option_bits, PLUGIN_VAR_NOSYSVAR | PLUGIN_VAR_NOCMDOPT,
-                              "Storage for thd->variables.option_bits. For internal usage.", NULL, NULL, 0, 0,
+                              "Storage for thd->variables.option_bits. For internal usage", NULL, NULL, 0, 0,
                               ~0U, 1);
 
 const char* mcs_select_handler_mode_values[] = {"OFF", "ON", "AUTO", NullS};
 
-static TYPELIB mcs_select_handler_mode_values_lib = {array_elements(mcs_select_handler_mode_values) - 1,
-                                                     "mcs_select_handler_mode_values",
-                                                     mcs_select_handler_mode_values, NULL};
+static TYPELIB mcs_select_handler_mode_values_lib = DEFINE_TYPELIB(mcs_select_handler_mode_values);
 
 static MYSQL_THDVAR_ENUM(select_handler, PLUGIN_VAR_RQCMDARG,
                          "Set the MCS select_handler to Disabled, Enabled, or Automatic",
@@ -78,9 +91,6 @@ static MYSQL_THDVAR_ENUM(select_handler, PLUGIN_VAR_RQCMDARG,
 static MYSQL_THDVAR_BOOL(derived_handler, PLUGIN_VAR_NOCMDARG, "Enable/Disable the MCS derived_handler", NULL,
                          NULL, 1);
 
-static MYSQL_THDVAR_BOOL(group_by_handler, PLUGIN_VAR_NOCMDARG, "Enable/Disable the MCS group_by_handler",
-                         NULL, NULL, 1);
-
 static MYSQL_THDVAR_BOOL(select_handler_in_stored_procedures, PLUGIN_VAR_NOCMDARG,
                          "Enable/Disable the MCS select_handler for Stored Procedures", NULL, NULL, 1);
 
@@ -88,13 +98,19 @@ static MYSQL_THDVAR_UINT(orderby_threads, PLUGIN_VAR_RQCMDARG,
                          "Number of parallel threads used by ORDER BY. (default to 16)", NULL, NULL, 16, 0,
                          2048, 1);
 
+static constexpr uint DEFAULT_CES_OPTIMIZATION_PARALLEL_FACTOR = 50;
+
+static MYSQL_THDVAR_UINT(query_accel_parallel_factor, PLUGIN_VAR_RQCMDARG,
+                         "Maximum parallel factor for parallel CES optimization. (default to 50)", NULL, NULL,
+                         DEFAULT_CES_OPTIMIZATION_PARALLEL_FACTOR, 1, 1000, 1);
+
 // legacy system variables
 static MYSQL_THDVAR_ULONG(decimal_scale, PLUGIN_VAR_RQCMDARG,
-                          "The default decimal precision for calculated column sub-operations ", NULL, NULL,
+                          "The default decimal precision for calculated column sub-operations", NULL, NULL,
                           8, 0, 18, 1);
 
 static MYSQL_THDVAR_BOOL(varbin_always_hex, PLUGIN_VAR_NOCMDARG,
-                         "Always display/process varbinary columns as if they have been hexified.", NULL,
+                         "Always display/process varbinary columns as if they have been hexified", NULL,
                          NULL, 0);
 
 static MYSQL_THDVAR_BOOL(use_decimal_scale, PLUGIN_VAR_NOCMDARG,
@@ -102,14 +118,14 @@ static MYSQL_THDVAR_BOOL(use_decimal_scale, PLUGIN_VAR_NOCMDARG,
 
 static MYSQL_THDVAR_BOOL(
     double_for_decimal_math, PLUGIN_VAR_NOCMDARG,
-    "Enable/disable for ColumnStore to replace DECIMAL with DOUBLE in arithmetic operation.", NULL, NULL, 0);
+    "Enable/disable for ColumnStore to replace DECIMAL with DOUBLE in arithmetic operation", NULL, NULL, 0);
 
 static MYSQL_THDVAR_BOOL(decimal_overflow_check, PLUGIN_VAR_NOCMDARG,
-                         "Enable/disable for ColumnStore to check for overflow in arithmetic operation.",
+                         "Enable/disable for ColumnStore to check for overflow in arithmetic operation",
                          NULL, NULL, 0);
 
 static MYSQL_THDVAR_BOOL(ordered_only, PLUGIN_VAR_NOCMDARG,
-                         "Always use the first table in the from clause as the large side "
+                         "Always use the first table in the from clause as the large side"
                          "table for joins",
                          NULL, NULL, 0);
 
@@ -122,25 +138,35 @@ static MYSQL_THDVAR_ULONG(stringtable_threshold, PLUGIN_VAR_RQCMDARG,
                           20, 9, ~0U, 1);
 
 static MYSQL_THDVAR_ULONG(diskjoin_smallsidelimit, PLUGIN_VAR_RQCMDARG,
-                          "The maximum amount of disk space in MB to use per query for storing "
+                          "The maximum amount of disk space in MB to use per query for storing"
                           "'small side' tables for a disk-based join. (0 = unlimited)",
                           NULL, NULL, 0, 0, ~0U, 1);
 
 static MYSQL_THDVAR_ULONG(diskjoin_largesidelimit, PLUGIN_VAR_RQCMDARG,
-                          "The maximum amount of disk space in MB to use per join for storing "
+                          "The maximum amount of disk space in MB to use per join for storing"
                           "'large side' table data for a disk-based join. (0 = unlimited)",
                           NULL, NULL, 0, 0, ~0U, 1);
 
 static MYSQL_THDVAR_ULONG(diskjoin_bucketsize, PLUGIN_VAR_RQCMDARG,
-                          "The maximum size in MB of each 'small side' table in memory.", NULL, NULL, 100, 1,
+                          "The maximum size in MB of each 'small side' table in memory", NULL, NULL, 100, 1,
                           ~0U, 1);
 
+static MYSQL_THDVAR_ULONG(diskjoin_max_partition_tree_depth, PLUGIN_VAR_RQCMDARG,
+                          "The maximum size of partition tree depth", NULL, NULL, 8, 1, ~0U, 1);
+
+static MYSQL_THDVAR_BOOL(diskjoin_force_run, PLUGIN_VAR_RQCMDARG, "Force run for the disk join step", NULL,
+                         NULL, 0);
+
+static MYSQL_THDVAR_ULONG(max_pm_join_result_count, PLUGIN_VAR_RQCMDARG,
+                          "The maximum size of the join result for the single block on BPP", NULL, NULL,
+                          1048576, 1, ~0U, 1);
+
 static MYSQL_THDVAR_ULONG(um_mem_limit, PLUGIN_VAR_RQCMDARG,
                           "Per user Memory limit(MB). Switch to disk-based JOIN when limit is reached", NULL,
                           NULL, 0, 0, ~0U, 1);
 
 static MYSQL_THDVAR_ULONG(local_query, PLUGIN_VAR_RQCMDARG,
-                          "Enable/disable the ColumnStore local PM query only feature.", NULL, NULL, 0, 0, 2,
+                          "Enable/disable the ColumnStore local PM query only feature", NULL, NULL, 0, 0, 2,
                           1);
 
 static MYSQL_THDVAR_ULONG(import_for_batchinsert_delimiter, PLUGIN_VAR_RQCMDARG,
@@ -165,9 +191,8 @@ static MYSQL_THDVAR_ULONG(import_for_batchinsert_enclosed_by, PLUGIN_VAR_RQCMDAR
 
 const char* mcs_use_import_for_batchinsert_mode_values[] = {"OFF", "ON", "ALWAYS", NullS};
 
-static TYPELIB mcs_use_import_for_batchinsert_mode_values_lib = {
-    array_elements(mcs_use_import_for_batchinsert_mode_values) - 1,
-    "mcs_use_import_for_batchinsert_mode_values", mcs_use_import_for_batchinsert_mode_values, NULL};
+static TYPELIB mcs_use_import_for_batchinsert_mode_values_lib =
+  DEFINE_TYPELIB(mcs_use_import_for_batchinsert_mode_values);
 
 static MYSQL_THDVAR_ENUM(use_import_for_batchinsert, PLUGIN_VAR_RQCMDARG,
                          "LOAD DATA INFILE and INSERT..SELECT will use cpimport internally",
@@ -191,15 +216,44 @@ static MYSQL_THDVAR_ULONGLONG(cache_flush_threshold, PLUGIN_VAR_RQCMDARG,
                               "Threshold on the number of rows in the cache to trigger a flush", NULL, NULL,
                               500000, 1, 1000000000, 1);
 
+static MYSQL_THDVAR_STR(cmapi_host, PLUGIN_VAR_NOCMDOPT | PLUGIN_VAR_MEMALLOC, "CMAPI host", NULL, NULL,
+                        "https://localhost");
+
+static MYSQL_THDVAR_STR(cmapi_version, PLUGIN_VAR_NOCMDOPT | PLUGIN_VAR_MEMALLOC, "CMAPI version", NULL, NULL,
+                        "0.4.0");
+
+static MYSQL_THDVAR_STR(cmapi_key, PLUGIN_VAR_NOCMDOPT | PLUGIN_VAR_MEMALLOC, "CMAPI key", NULL, NULL, "");
+static MYSQL_THDVAR_STR(pron, PLUGIN_VAR_NOCMDOPT | PLUGIN_VAR_MEMALLOC, "Debug options json dictionary",
+                        NULL, NULL, "");
+
+static MYSQL_THDVAR_ULONGLONG(cmapi_port, PLUGIN_VAR_NOCMDOPT, "CMAPI port", NULL, NULL, 8640, 100, 65356, 1);
+
+static MYSQL_THDVAR_STR(s3_key, PLUGIN_VAR_NOCMDOPT | PLUGIN_VAR_MEMALLOC, "S3 Authentication Key", NULL,
+                        NULL, "");
+static MYSQL_THDVAR_STR(s3_secret, PLUGIN_VAR_NOCMDOPT | PLUGIN_VAR_MEMALLOC, "S3 Authentication Secret",
+                        NULL, NULL, "");
+static MYSQL_THDVAR_STR(s3_region, PLUGIN_VAR_NOCMDOPT | PLUGIN_VAR_MEMALLOC, "S3 region", NULL, NULL, "");
+
+static MYSQL_THDVAR_ULONG(max_allowed_in_values, PLUGIN_VAR_RQCMDARG,
+                          "The maximum length of the entries in the IN query clause", NULL, NULL, 6000, 1,
+                          ~0U, 1);
+
+static my_bool innodb_queries_use_mcs;
+static MYSQL_SYSVAR_BOOL(innodb_queries_use_mcs, innodb_queries_use_mcs,
+                      PLUGIN_VAR_OPCMDARG | PLUGIN_VAR_READONLY,
+                      "Direct all InnoDB-only queries into MCS via Select Handler", NULL, NULL, FALSE);
+static MYSQL_THDVAR_BOOL(unstable_optimizer, PLUGIN_VAR_RQCMDARG,
+                         "Apply optimizer rules after translation from SELECT_LEX/UNION", NULL, NULL, FALSE);
+
 st_mysql_sys_var* mcs_system_variables[] = {MYSQL_SYSVAR(compression_type),
                                             MYSQL_SYSVAR(fe_conn_info_ptr),
                                             MYSQL_SYSVAR(original_optimizer_flags),
                                             MYSQL_SYSVAR(original_option_bits),
                                             MYSQL_SYSVAR(select_handler),
                                             MYSQL_SYSVAR(derived_handler),
-                                            MYSQL_SYSVAR(group_by_handler),
                                             MYSQL_SYSVAR(select_handler_in_stored_procedures),
                                             MYSQL_SYSVAR(orderby_threads),
+                                            MYSQL_SYSVAR(query_accel_parallel_factor),
                                             MYSQL_SYSVAR(decimal_scale),
                                             MYSQL_SYSVAR(use_decimal_scale),
                                             MYSQL_SYSVAR(ordered_only),
@@ -208,6 +262,9 @@ st_mysql_sys_var* mcs_system_variables[] = {MYSQL_SYSVAR(compression_type),
                                             MYSQL_SYSVAR(diskjoin_smallsidelimit),
                                             MYSQL_SYSVAR(diskjoin_largesidelimit),
                                             MYSQL_SYSVAR(diskjoin_bucketsize),
+                                            MYSQL_SYSVAR(diskjoin_max_partition_tree_depth),
+                                            MYSQL_SYSVAR(diskjoin_force_run),
+                                            MYSQL_SYSVAR(max_pm_join_result_count),
                                             MYSQL_SYSVAR(um_mem_limit),
                                             MYSQL_SYSVAR(double_for_decimal_math),
                                             MYSQL_SYSVAR(decimal_overflow_check),
@@ -220,6 +277,17 @@ st_mysql_sys_var* mcs_system_variables[] = {MYSQL_SYSVAR(compression_type),
                                             MYSQL_SYSVAR(cache_inserts),
                                             MYSQL_SYSVAR(cache_use_import),
                                             MYSQL_SYSVAR(cache_flush_threshold),
+                                            MYSQL_SYSVAR(cmapi_host),
+                                            MYSQL_SYSVAR(cmapi_port),
+                                            MYSQL_SYSVAR(cmapi_version),
+                                            MYSQL_SYSVAR(cmapi_key),
+                                            MYSQL_SYSVAR(s3_key),
+                                            MYSQL_SYSVAR(s3_secret),
+                                            MYSQL_SYSVAR(s3_region),
+                                            MYSQL_SYSVAR(pron),
+                                            MYSQL_SYSVAR(max_allowed_in_values),
+                                            MYSQL_SYSVAR(innodb_queries_use_mcs),
+                                            MYSQL_SYSVAR(unstable_optimizer),
                                             NULL};
 
 st_mysql_show_var mcs_status_variables[] = {{"columnstore_version", (char*)&cs_version, SHOW_CHAR},
@@ -233,13 +301,15 @@ void* get_fe_conn_info_ptr(THD* thd)
 
 void set_fe_conn_info_ptr(void* ptr, THD* thd)
 {
-  if (thd == NULL) thd = current_thd;
-  if (thd == NULL) return;
+  if (thd == NULL)
+    thd = current_thd;
+  if (thd == NULL)
+    return;
 
   THDVAR(thd, fe_conn_info_ptr) = (uint64_t)(ptr);
 }
 
-ulonglong get_original_optimizer_flags(THD* thd)
+ulonglong get_original_optimizer_flags(THD* /*thd*/)
 {
   return THDVAR(current_thd, original_optimizer_flags);
 }
@@ -288,15 +358,6 @@ void set_derived_handler(THD* thd, bool value)
   THDVAR(thd, derived_handler) = value;
 }
 
-bool get_group_by_handler(THD* thd)
-{
-  return (thd == NULL) ? false : THDVAR(thd, group_by_handler);
-}
-void set_group_by_handler(THD* thd, bool value)
-{
-  THDVAR(thd, group_by_handler) = value;
-}
-
 bool get_select_handler_in_stored_procedures(THD* thd)
 {
   return (thd == NULL) ? true : THDVAR(thd, select_handler_in_stored_procedures);
@@ -325,6 +386,15 @@ void set_orderby_threads(THD* thd, uint value)
   THDVAR(thd, orderby_threads) = value;
 }
 
+uint get_query_accel_parallel_factor(THD* thd)
+{
+  return (thd == NULL) ? DEFAULT_CES_OPTIMIZATION_PARALLEL_FACTOR : THDVAR(thd, query_accel_parallel_factor);
+}
+void set_query_accel_parallel_factor(THD* thd, uint value)
+{
+  THDVAR(thd, query_accel_parallel_factor) = value;
+}
+
 bool get_use_decimal_scale(THD* thd)
 {
   return (thd == NULL) ? false : THDVAR(thd, use_decimal_scale);
@@ -397,6 +467,33 @@ void set_diskjoin_bucketsize(THD* thd, ulong value)
   THDVAR(thd, diskjoin_bucketsize) = value;
 }
 
+ulong get_diskjoin_max_partition_tree_depth(THD* thd)
+{
+  return (thd == NULL) ? 0 : THDVAR(thd, diskjoin_max_partition_tree_depth);
+}
+void set_diskjoin_max_partition_tree_depth(THD* thd, ulong value)
+{
+  THDVAR(thd, diskjoin_max_partition_tree_depth) = value;
+}
+
+bool get_diskjoin_force_run(THD* thd)
+{
+  return (thd == NULL) ? 0 : THDVAR(thd, diskjoin_force_run);
+}
+void set_diskjoin_force_run(THD* thd, bool value)
+{
+  THDVAR(thd, diskjoin_force_run) = value;
+}
+
+ulong get_max_pm_join_result_count(THD* thd)
+{
+  return (thd == NULL) ? 0 : THDVAR(thd, max_pm_join_result_count);
+}
+void set_max_pm_join_result_count(THD* thd, ulong value)
+{
+  THDVAR(thd, max_pm_join_result_count) = value;
+}
+
 ulong get_um_mem_limit(THD* thd)
 {
   return (thd == NULL) ? 0 : THDVAR(thd, um_mem_limit);
@@ -506,3 +603,108 @@ void set_cache_flush_threshold(THD* thd, ulonglong value)
 {
   THDVAR(thd, cache_flush_threshold) = value;
 }
+
+ulong get_cmapi_port(THD* thd)
+{
+  return (thd == NULL) ? 5000 : THDVAR(thd, cmapi_port);
+}
+
+void set_cmapi_port(THD* thd, ulong value)
+{
+  THDVAR(thd, cmapi_port) = value;
+}
+
+const char* get_cmapi_host(THD* thd)
+{
+  return (thd == NULL) ? "127.0.0.1" : THDVAR(thd, cmapi_host);
+}
+
+void set_cmapi_host(THD* thd, char* value)
+{
+  THDVAR(thd, cmapi_host) = value;
+}
+
+const char* get_cmapi_version(THD* thd)
+{
+  return (thd == NULL) ? "0.4.0" : THDVAR(thd, cmapi_version);
+}
+
+void set_cmapi_version(THD* thd, char* value)
+{
+  THDVAR(thd, cmapi_version) = value;
+}
+
+const char* get_cmapi_key(THD* thd)
+{
+  return (thd == NULL) ? "" : THDVAR(thd, cmapi_key);
+}
+
+void set_cmapi_key(THD* thd, char* value)
+{
+  THDVAR(thd, cmapi_key) = value;
+}
+
+const char* get_pron(THD* thd)
+{
+  return (thd == NULL) ? "" : THDVAR(thd, pron);
+}
+
+void set_pron(THD* thd, char* value)
+{
+  THDVAR(thd, pron) = value;
+}
+
+const char* get_s3_key(THD* thd)
+{
+  return THDVAR(thd, s3_key);
+}
+
+void set_s3_key(THD* thd, char* value)
+{
+  THDVAR(thd, s3_key) = value;
+}
+
+const char* get_s3_secret(THD* thd)
+{
+  return THDVAR(thd, s3_secret);
+}
+
+void set_s3_secret(THD* thd, char* value)
+{
+  THDVAR(thd, s3_secret) = value;
+}
+
+const char* get_s3_region(THD* thd)
+{
+  return THDVAR(thd, s3_region);
+}
+
+void set_s3_region(THD* thd, char* value)
+{
+  THDVAR(thd, s3_region) = value;
+}
+
+ulong get_max_allowed_in_values(THD* thd)
+{
+  return (thd == NULL) ? 0 : THDVAR(thd, max_allowed_in_values);
+}
+void set_max_allowed_in_values(THD* thd, ulong value)
+{
+  THDVAR(thd, max_allowed_in_values) = value;
+}
+
+bool get_unstable_optimizer(THD* thd)
+{
+  return (thd == NULL) ? 0 : THDVAR(thd, unstable_optimizer);
+}
+void set_unstable_optimizer(THD* thd, bool value)
+{
+  THDVAR(thd, unstable_optimizer) = value;
+}
+
+bool get_innodb_queries_uses_mcs()
+{
+  return SYSVAR(innodb_queries_use_mcs);
+}
+
+#undef DEFINE_TYPELIB
\ No newline at end of file
