@@ -908,7 +908,7 @@ static inline void fil_crypt_read_crypt_data(fil_space_t *space)
     return;
 
   const ulint zip_size= space->zip_size();
-  mtr_t mtr;
+  mtr_t mtr{nullptr};
   mtr.start();
   if (buf_block_t* b= buf_page_get_gen(page_id_t{space->id, 0}, zip_size,
                                        RW_S_LATCH, nullptr,
@@ -962,7 +962,7 @@ func_exit:
 	fil_crypt_start_converting = true;
 	mysql_mutex_unlock(&fil_crypt_threads_mutex);
 
-	mtr_t mtr;
+	mtr_t mtr{nullptr};
 	mtr.start();
 
 	/* 2 - get page 0 */
@@ -1687,7 +1687,7 @@ fil_crypt_get_page_throttle(
 
 	if (offset % (zip_size ? zip_size : srv_page_size)
 	    && DB_SUCCESS_LOCKED_REC
-	    != fseg_page_is_allocated(space, offset)) {
+	    != fseg_page_is_allocated(mtr, space, offset)) {
 		/* page is already freed */
 		return NULL;
 	}
@@ -1755,7 +1755,7 @@ fil_crypt_rotate_page(
 		return;
 	}
 
-	mtr_t mtr;
+	mtr_t mtr{nullptr};
 	mtr.start();
 	if (buf_block_t* block = fil_crypt_get_page_throttle(state,
 							     offset, &mtr,
@@ -1936,7 +1936,7 @@ fil_crypt_flush_space(
 	}
 
 	/* update page 0 */
-	mtr_t mtr;
+	mtr_t mtr{nullptr};
 	mtr.start();
 
 	if (buf_block_t* block = buf_page_get_gen(
@@ -2113,6 +2113,9 @@ Adjust thread count for key rotation
 @param[in]	enw_cnt		Number of threads to be used */
 void fil_crypt_set_thread_cnt(const uint new_cnt)
 {
+	if (srv_read_only_mode)
+		return;
+
 	if (!fil_crypt_threads_inited) {
 		if (srv_shutdown_state != SRV_SHUTDOWN_NONE)
 			return;
@@ -2266,6 +2269,8 @@ void fil_crypt_set_encrypt_tables(ulong val)
 Init threads for key rotation */
 void fil_crypt_threads_init()
 {
+	ut_ad(!srv_read_only_mode);
+
 	if (!fil_crypt_threads_inited) {
 		pthread_cond_init(&fil_crypt_cond, nullptr);
 		pthread_cond_init(&fil_crypt_threads_cond, nullptr);
